"""Utility implementations for docstrings.
"""

import inspect
import os
import re
from enum import Enum
from inspect import Signature
from typing import Any
from typing import Callable
from typing import List
from typing import Match
from typing import Optional
from typing import Pattern
from typing import Tuple
from typing import Type
from typing import TypeVar

DOCSTRING_PATH_COMMENT_KEYWORD: str = 'Docstring:'
DOCSTRING_PATH_COMMENT_PATTERN: str = (
    rf'^\<\!\-\-.*?{DOCSTRING_PATH_COMMENT_KEYWORD}'
    r'(?P<path>.*?)\-\-\>'
)

_HYPHENS_LINE_PATTERN: str = r'(\s{4}|^)-----'


class _SectionPattern(Enum):
    PARAMETERS = r'(\s{4}|^)Parameters$'
    RETURNS = r'(\s{4}|^)Returns$'
    YIELDS = r'(\s{4}|^)Yields$'
    RECEIVES = r'(\s{4}|^)Receives$'
    RAISES = r'(\s{4}|^)Raises$'
    WARNS = r'(\s{4}|^)Warns$'
    WARNINGS = r'(\s{4}|^)Warnings$'
    SEE_ALSO = r'(\s{4}|^)See Also$'
    NOTES = r'(\s{4}|^)Notes$'
    REFERENCES = r'(\s{4}|^)References$'
    EXAMPLES = r'(\s{4}|^)Examples$'
    ATTRIBUTES = r'(\s{4}|^)Attributes$'
    METHODS = r'(\s{4}|^)Methods$'


def get_docstring_src_module_paths(md_file_path: str) -> List[str]:
    """
    Get docstring source module paths from a specified
    markdown file path.

    Parameters
    ----------
    md_file_path : str
        Target markdown file path.

    Returns
    -------
    module_paths : list of str
        Extracted docstring source module paths.
    """
    from apysc._file import file_util
    from apysc._file import module_util
    md_txt: str = file_util.read_txt(file_path=md_file_path)
    lines: List[str] = md_txt.splitlines()
    module_paths: List[str] = []
    pattern: Pattern = re.compile(
        pattern=DOCSTRING_PATH_COMMENT_PATTERN)
    for line in lines:
        if not line.startswith('<!--'):
            continue
        match: Optional[Match] = pattern.search(string=line)
        if match is None:
            continue
        package_path: str = match.group(1).rsplit('.', maxsplit=1)[0]
        package_path = package_path.strip()
        module_or_class: Any = module_util.\
            read_module_or_class_from_package_path(
                module_or_class_package_path=package_path)
        if inspect.isclass(module_or_class):
            package_path = package_path.rsplit('.', maxsplit=1)[0]
        module_path: str = package_path.replace('.', '/')
        module_path = f'./{module_path}.py'
        module_paths.append(module_path)
    return module_paths


def reset_replaced_docstring_section(*, md_file_path: str) -> bool:
    """
    Reset converted a markdown's docstring section.

    Parameters
    ----------
    md_file_path : str
        Target markdown document file path.

    Returns
    -------
    is_executed : bool
        Replacing is executed or not.
    """
    from apysc._file import file_util
    md_txt: str = file_util.read_txt(file_path=md_file_path)
    matches: List[str] = _get_docstring_path_comment_matches(md_txt=md_txt)
    if not matches:
        return False
    md_txt = _remove_replaced_docstring_section_from_md_txt(
        md_txt=md_txt, matches=matches)
    with open(md_file_path, 'w') as f:
        f.write(md_txt)
    return True


def _remove_replaced_docstring_section_from_md_txt(
        *, md_txt: str, matches: List[str]) -> str:
    """
    Remove replaced docstring from a specified markdown text.

    Parameters
    ----------
    md_txt : str
        Target markdown text.
    matches : list of str
        Matched docstring path specification comments.

    Returns
    -------
    md_txt : str
        Result markdown text.
    """
    lines: List[str] = md_txt.splitlines()
    result_lines: List[str] = []
    is_reset_section_range: bool = False
    for line in lines:
        if is_reset_section_range:
            if line.startswith('#'):
                result_lines.append(f'\n{line}')
                is_reset_section_range = False
            continue
        docstring_path_specification_comment: str = \
            _extract_docstring_path_specification_comment_from_line(
                line=line, matches=matches)
        if docstring_path_specification_comment != '':
            result_lines.append(line)
            is_reset_section_range = True
            continue
        result_lines.append(line)
    md_txt = '\n'.join(result_lines)
    return md_txt


def _extract_docstring_path_specification_comment_from_line(
        *, line: str, matches: List[str]) -> str:
    """
    Extract a docstring path specification comment
    from a specified markdown line text.

    Parameters
    ----------
    line : str
        Target markdown line text.
    matches : list of str
        Matched docstring path specification comments.

    Returns
    -------
    docstring_path_specification_comment : str
        Extracted comment string.
    """
    for match in matches:
        if match in line:
            return match
    return ''


def _get_docstring_path_comment_matches(*, md_txt: str) -> List[str]:
    """
    Get matched docstring path specification comments.

    Parameters
    ----------
    md_txt : str
        Target markdown text.

    Returns
    -------
    matches : list of str
        Matched comments.
    """
    matches: List[str] = []
    for match in re.finditer(
            pattern=DOCSTRING_PATH_COMMENT_PATTERN,
            string=md_txt,
            flags=re.MULTILINE):
        matches.append(match.group(0))
    return matches


def replace_docstring_path_specification(*, md_file_path: str) -> None:
    """
    Replace a docstring path specification with a converted
    docstring text in a specified markdown document.

    Parameters
    ----------
    md_file_path : str
        Target markdown file path.
    """
    from apysc._file import file_util
    md_txt: str = file_util.read_txt(file_path=md_file_path)
    lines: List[str] = md_txt.splitlines()
    result_lines: List[str] = []
    for line in lines:
        match: Optional[Match] = re.search(
            pattern=DOCSTRING_PATH_COMMENT_PATTERN, string=line)
        if match is not None:
            result_lines.append(line)
            result_lines.append('')
            markdown_format_docstring: str = \
                _convert_docstring_path_comment_to_markdown_format(
                    docstring_path_comment=match.group(0),
                    md_file_path=md_file_path,
                )
            result_lines.append(markdown_format_docstring)
            continue

        result_lines.append(line)
        continue
    md_txt = '\n'.join(result_lines)
    file_util.save_plain_txt(
        txt=md_txt, file_path=md_file_path)


class _DocstringPathNotFoundError(Exception):
    pass


class _DocstringCallableNotExistsError(Exception):
    pass


def _convert_docstring_path_comment_to_markdown_format(
        *, docstring_path_comment: str,
        md_file_path: str) -> str:
    """
    Convert a specified docstring path comment to a
    markdown format text.

    Parameters
    ----------
    docstring_path_comment : str
        Target docstring path comment.
    md_file_path : str
        Target markdown file path.

    Returns
    -------
    markdown_format_docstring : str
        Converted text.
    """
    module_or_class_package_path: str
    callable_name: str
    module_or_class_package_path, callable_name = \
        _extract_package_path_and_callable_name_from_path(
            docstring_path_comment=docstring_path_comment,
        )
    callable_: Callable = _get_callable_from_package_path_and_callable_name(
        module_or_class_package_path=module_or_class_package_path,
        callable_name=callable_name,
    )
    if callable_.__doc__ is None:
        return ''
    if callable(callable_):
        signature: Optional[Signature] = inspect.signature(callable_)
        callable_name = callable_.__name__
    else:
        signature = None
        callable_name = ''
    markdown_format_docstring: str = _convert_docstring_to_markdown(
        docstring=callable_.__doc__,
        signature=signature,
        callable_name=callable_name,
        md_file_path=md_file_path)
    return markdown_format_docstring


def _get_callable_from_package_path_and_callable_name(
        *, module_or_class_package_path: str,
        callable_name: str) -> Callable:
    """
    Get a callable object from a specified package path and
    callable name.

    Parameters
    ----------
    module_or_class_package_path : str
        Target module or class package path.
    callable_name : str
        Target callable name.

    Raises
    ------
    _DocstringPathNotFoundError
        If a specified package path's module or class
        does not exist.
    _DocstringCallableNotExistsError
        If a target module or class does not have a specified
        name function or method.

    Returns
    -------
    callable_ : Callable
        Target callable object.
    """
    from apysc._file import module_util
    try:
        module_or_class: Any = \
            module_util.read_module_or_class_from_package_path(
                module_or_class_package_path=module_or_class_package_path)
    except Exception:
        raise _DocstringPathNotFoundError(
            'Could not found module or class of the docstring path.'
            f'\nModule or class package path: {module_or_class_package_path}'
        )
    try:
        callable_: Callable = getattr(module_or_class, callable_name)
    except Exception:
        raise _DocstringCallableNotExistsError(
            "Specified docstring path's module or class does not have "
            'a target callable attribute.'
            f'\nModule or class package path: {module_or_class_package_path}',
            f'\nCallable name: {callable_name}')
    return callable_


def _convert_docstring_to_markdown(
        *, docstring: str,
        signature: Optional[Signature],
        callable_name: str,
        md_file_path: str) -> str:
    """
    Convert a specified docstring to a markdown format text.

    Parameters
    ----------
    docstring : str
        Target docstring.
    signature : Signature or None
        Target callable's signature. If a target interface
        is property, this argument becomes None.
    callable_name : str
        Target callable name.
    md_file_path : str
        Target markdown file path.

    Returns
    -------
    markdown : str
        Converted markdown text.
    """
    summary: str = extract_summary_from_docstring(docstring=docstring)
    parameters: List[Parameter] = \
        extract_param_or_rtn_values_from_docstring(
            target_type=Parameter, docstring=docstring)
    returns: List[Return] = extract_param_or_rtn_values_from_docstring(
        target_type=Return, docstring=docstring)
    raises: List[Raise] = extract_raise_values_from_docstring(
        docstring=docstring)
    notes: str = extract_notes_from_docstring(docstring=docstring)
    examples: List[Example] = extract_example_values_from_docstring(
        docstring=docstring)
    references: List[Reference] = extract_reference_values_from_docstring(
        docstring=docstring)
    references = _slice_references_by_md_file_path(
        references=references, md_file_path=md_file_path)
    markdown: str = (
        '<span class="inconspicuous-txt">Note: the document '
        'build script generates and updates this '
        'API document section automatically. Maybe this section '
        'is duplicated compared with previous sections.</span>')
    if signature is not None:
        markdown += (
            f'\n\n**[Interface signature]** `{callable_name}{signature}`<hr>'
        )
    markdown = append_summary_to_markdown(
        markdown=markdown, summary=summary,
        heading_label='**[Interface summary]** ')
    markdown = append_params_or_rtns_to_markdown(
        markdown=markdown, params_or_rtns=parameters)
    markdown = append_params_or_rtns_to_markdown(
        markdown=markdown, params_or_rtns=returns)
    markdown = append_raises_to_markdown(markdown=markdown, raises=raises)
    markdown = append_notes_to_markdown(markdown=markdown, notes=notes)
    markdown = append_examples_to_markdown(
        markdown=markdown, examples=examples)
    markdown = append_references_to_markdown(
        markdown=markdown, references=references)
    markdown = remove_trailing_hr_tag(markdown=markdown)
    return markdown


def remove_trailing_hr_tag(*, markdown: str) -> str:
    """
    Remove a trailing `<hr>` tag from a specified markdown string.

    Parameters
    ----------
    markdown : str
        Target markdown string.

    Returns
    -------
    markdown : str
        A result markdown string.
    """
    markdown = markdown.strip()
    hr_tag: str = '<hr>'
    if markdown.endswith(hr_tag):
        markdown = markdown[:-len(hr_tag)]
    markdown = markdown.strip()
    return markdown


def append_summary_to_markdown(
        *, markdown: str, summary: str,
        heading_label: str) -> str:
    """
    Append an interface summary string to a specified
    markdown string.

    Parameters
    ----------
    markdown : str
        Target markdown string.
    summary : str
        Target summary string.
    heading_label : str
        A label to append at the beginning.

    Returns
    -------
    markdown : str
        A result markdown string.
    """
    if summary == '':
        return markdown
    if markdown != '':
        markdown += '\n\n'
    if summary.strip().startswith('<br>'):
        summary = summary.replace('<br>', '', 1)
    markdown += f'{heading_label}{summary}<hr>'
    return markdown


def append_notes_to_markdown(*, markdown: str, notes: str) -> str:
    """
    Append a notes string to a specified markdown string.

    Parameters
    ----------
    markdown : str
        Target markdown string.
    notes : str
        Target notes string.

    Returns
    -------
    markdown : str
        A result markdown string.
    """
    if notes == '':
        return markdown
    if markdown != '':
        markdown += '\n\n'
    if notes.strip().startswith('<br>'):
        notes = notes.replace('<br>', '', 1)
    markdown += (
        '**[Notes]**'
        f'\n\n{notes}<hr>'
    )
    return markdown


def extract_notes_from_docstring(*, docstring: str) -> str:
    """
    Extract a notes' value from a docstring.

    Parameters
    ----------
    docstring : str
        Target docstring.

    Returns
    -------
    notes : str
        Extract notes text value.
    """
    lines: List[str] = docstring.splitlines()
    lines = _remove_blank_lines_from_list(lines=lines)
    is_notes_section_range: bool = False
    notes_lines: List[str] = []
    for line in lines:
        if _is_target_section_pattern_line(
                line=line,
                section_pattern=_SectionPattern.NOTES):
            is_notes_section_range = True
            continue
        if _is_skip_target_line(
                is_target_section_range=is_notes_section_range,
                line=line):
            continue
        if _is_section_line(line=line):
            break

        line = _append_br_tag_and_replace_symbol_if_first_char_is_hyphen(
            line=line)
        notes_lines.append(line)
    notes: str = '\n'.join(notes_lines)
    notes = _remove_line_breaks_and_unnecessary_spaces(text=notes)
    return notes


class _ParamOrRtnBase:
    _name: str
    _type_str: str
    _description: str

    def __init__(
            self, *, name: str, type_str: str, description: str) -> None:
        """
        Parameter or return value's base class.

        Parameters
        ----------
        name : str
            Parameter or return value name.
        type_str : str
            Parameter or return value type name.
        description : str
            Parameter or return value description.
        """
        self._name = name
        self._type_str = type_str
        self._description = description

    def __eq__(self, other: Any) -> bool:
        """
        The method for equality comparison.

        Parameters
        ----------
        other : Any
            The other instance to compare.

        Returns
        -------
        result : bool
            If each attribute is equal to the other, this
            method returns True.
        """
        if not isinstance(other, _ParamOrRtnBase):
            return False
        if self.name != other.name:
            return False
        if self.type_str != other.type_str:
            return False
        if self.description != other.description:
            return False
        return True

    @property
    def name(self) -> str:
        """
        Get a parameter or return value name.

        Returns
        -------
        name : str
            A parameter or return value name.
        """
        return self._name

    @property
    def type_str(self) -> str:
        """
        Get a parameter or return value type name.

        Returns
        -------
        type_str : str
            A parameter or return value type name.
        """
        return self._type_str

    @property
    def description(self) -> str:
        """
        Get a parameter or return value description.

        Returns
        -------
        description : str
            A parameter or return value description.
        """
        return self._description


class Parameter(_ParamOrRtnBase):
    """Parameter value type.
    """


class Return(_ParamOrRtnBase):
    """Return value type.
    """


class Raise:
    """Raise value type.
    """
    _err_class_name: str
    _description: str

    def __init__(self, *, err_class_name: str, description: str) -> None:
        """
        Raise value type.

        Parameters
        ----------
        err_class_name : str
            Target error class name.
        description : str
            Error condition description.
        """
        self._err_class_name = err_class_name
        self._description = description

    @property
    def err_class_name(self) -> str:
        """
        Get a target error class name.

        Returns
        -------
        err_class_name : str
            A target error class name.
        """
        return self._err_class_name

    @property
    def description(self) -> str:
        """
        Get a error condition description.

        Returns
        -------
        description : str
            A error condition description.
        """
        return self._description

    def __eq__(self, other: Any) -> bool:
        """
        The method for equality comparison.

        Parameters
        ----------
        other : Any
            The other value to compare.

        Returns
        -------
        result : bool
            If each attribute is equal to the other, this
            method returns True.
        """
        if not isinstance(other, Raise):
            return False
        if self.err_class_name != other.err_class_name:
            return False
        if self._description != other._description:
            return False
        return True


class Reference:
    """Reference value type.
    """

    _page_label: str
    _url: str

    def __init__(self, *, page_label: str, url: str) -> None:
        """
        Reference value type.

        Parameters
        ----------
        page_label : str
            Target reference page label.
        url : str
            Target reference page URL.
        """
        self._page_label = page_label
        self._url = url

    @property
    def page_label(self) -> str:
        """
        Get a target reference page label.

        Returns
        -------
        page_label : str
            A target reference page label.
        """
        return self._page_label

    @property
    def url(self) -> str:
        """
        Get a target reference page URL.

        Returns
        -------
        url : str
            A target reference page.
        """
        return self._url

    def __eq__(self, other: Any) -> bool:
        """
        The method for equality comparison.

        Parameters
        ----------
        other : Any
            The other value to compare.

        Returns
        -------
        result : bool
            If each attribute is equal to the other, this
            method returns True.
        """
        if not isinstance(other, Reference):
            return False
        if self.page_label != other.page_label:
            return False
        if self.url != other.url:
            return False
        return True


class Example:
    """Example value type.
    """

    _input_code_block: str
    _expected_output: str

    def __init__(
            self, *, input_code_block: str,
            expected_output: str = '') -> None:
        """
        Example value type.

        Parameters
        ----------
        input_code_block : str
            Input code block string.
        expected_output : str, default ''
            Expected output string.
        """
        self._input_code_block = input_code_block
        self._expected_output = expected_output

    @property
    def input_code_block(self) -> str:
        """
        Get a input code block string.

        Returns
        -------
        input_code_block : str
            A input code block string.
        """
        return self._input_code_block

    @property
    def expected_output(self) -> str:
        """
        Get a expected output string.

        Returns
        -------
        expected_output : str
            A expected output string.
        """
        return self._expected_output

    def __eq__(self, other: Any) -> bool:
        """
        The method for equality comparison.

        Parameters
        ----------
        other : Any
            The other value to compare.

        Returns
        -------
        result : bool
            If each attribute is equal to the other, this
            method returns True.
        """
        if not isinstance(other, Example):
            return False
        if self.input_code_block != other.input_code_block:
            return False
        if self.expected_output != other.expected_output:
            return False
        return True


def _slice_references_by_md_file_path(
        references: List[Reference],
        md_file_path: str) -> List[Reference]:
    """
    Slice a specified references list to exclude the
    same URL's document file.

    Parameters
    ----------
    references : list of Reference
        Target references list to slice.
    md_file_path : str
        Target markdown file path.

    Returns
    -------
    sliced_references : list of Reference
        Sliced list.
    """
    md_file_name: str = os.path.basename(md_file_path)
    md_file_name = md_file_name.rsplit('.', maxsplit=1)[0]
    sliced_references: List[Reference] = []
    for reference in references:
        reference_file_name: str = reference.url.rsplit('/', 1)[-1]
        reference_file_name = reference_file_name.rsplit(
            '.', maxsplit=1)[0]
        if reference_file_name == md_file_name:
            continue
        sliced_references.append(reference)
    return sliced_references


def append_examples_to_markdown(
        *, markdown: str, examples: List[Example]) -> str:
    """
    Append examples to a specified markdown string.

    Parameters
    ----------
    markdown : str
        Target markdown string.
    examples : list of Example
        Examples list value to append.

    Returns
    -------
    markdown : str
        A result markdown string.
    """
    if not examples:
        return markdown
    if markdown != '':
        markdown += '\n\n'
    markdown += '**[Examples]**\n\n```py'
    for i, example in enumerate(examples):
        if i != 0:
            markdown += '\n'
        markdown += (
            f'\n{example.input_code_block}'
        )
        if example.expected_output != '':
            markdown += f'\n{example.expected_output}'
    markdown += '\n```'
    markdown += '\n\n<hr>'
    return markdown


def extract_example_values_from_docstring(
        *, docstring: str) -> List[Example]:
    """
    Extract example values from a docstring.

    Parameters
    ----------
    docstring : str
        Target docstring.

    Returns
    -------
    example_values : list of Example
        Extracted example values.
    """
    lines: List[str] = docstring.splitlines()
    lines = _remove_blank_lines_from_list(lines=lines)
    is_example_section_range: bool = False
    input_code_block_lines: List[str] = []
    example_values: List[Example] = []
    for line in lines:
        if _is_target_section_pattern_line(
                line=line,
                section_pattern=_SectionPattern.EXAMPLES):
            is_example_section_range = True
            continue
        if _is_skip_target_line(
                is_target_section_range=is_example_section_range,
                line=line):
            continue
        if _is_section_line(line=line):
            break
        if _is_example_output_line(line=line):
            _make_example_and_append_to_list(
                example_values=example_values,
                input_code_block_lines=input_code_block_lines,
                expected_output=line)
            continue
        input_code_block_lines.append(line)
    _make_example_and_append_to_list(
        example_values=example_values,
        input_code_block_lines=input_code_block_lines,
        expected_output='')
    return example_values


def _make_example_and_append_to_list(
        *,
        example_values: List[Example],
        input_code_block_lines: List[str],
        expected_output: str) -> None:
    """
    Make an example value and append it to a specified list.

    Notes
    -----
    This function clears a list of input code block lines.

    Parameters
    ----------
    example_values : list of Example
        A list to append an example value.
    input_code_block_lines : list of str
        A list of input code block lines.
    expected_output : str
        An expected output string.
    """
    if not input_code_block_lines:
        return
    input_code_block_lines_: List[str] = [
        line.strip() for line in input_code_block_lines]
    input_code_block: str = '\n'.join(input_code_block_lines_)
    example: Example = Example(
        input_code_block=input_code_block,
        expected_output=expected_output.strip())
    example_values.append(example)
    input_code_block_lines.clear()


def _is_example_output_line(*, line: str) -> bool:
    """
    Get a boolean indicating whether a specified line
    is an example section's output line or not.

    Parameters
    ----------
    line : str
        Target docstring line.

    Returns
    -------
    result : bool
        This function returns True if a specified line
        is an example section's output line.
    """
    line = line.strip()
    if line.startswith('>>>') or line.startswith('...'):
        return False
    return True


def append_references_to_markdown(
        markdown: str, references: List[Reference]) -> str:
    """
    Append references to a specified markdown string.

    Parameters
    ----------
    markdown : str
        Target markdown string.
    references : list of Reference
        References list value to append.

    Returns
    -------
    markdown : str
        A result markdown string.
    """
    if not references:
        return markdown
    if markdown != '':
        markdown += '\n\n'
    markdown += '**[References]**\n'
    for reference in references:
        markdown += (
            f'\n- [{reference.page_label}]({reference.url})'
        )
    markdown += '\n\n<hr>'
    return markdown


def append_raises_to_markdown(
        *, markdown: str, raises: List[Raise]) -> str:
    """
    Append raises to a specified markdown string.

    Parameters
    ----------
    markdown : str
        Target markdown string.
    raises : list of Raise
        Raises list value to append.

    Returns
    -------
    markdown : str
        A result markdown string.
    """
    if not raises:
        return markdown
    if markdown != '':
        markdown += '\n\n'
    markdown += '**[Raises]**\n'
    for raise_ in raises:
        markdown += (
            f'\n- {raise_.err_class_name}: {raise_.description}'
        )
    markdown += '\n\n<hr>'
    return markdown


_ParamOrRtn = TypeVar('_ParamOrRtn', Parameter, Return)


def append_params_or_rtns_to_markdown(
        *, markdown: str,
        params_or_rtns: List[_ParamOrRtn]) -> str:
    """
    Append parameters or returns to a specified markdown string.

    Parameters
    ----------
    markdown : str
        Target markdown string.
    params_or_rtns : list of _ParamOrRtn
        Parameters or returns to append to.

    Returns
    -------
    markdown : str
        A result markdown string.
    """
    if not params_or_rtns:
        return markdown
    if isinstance(params_or_rtns[0], Parameter):
        section_label: str = 'Parameters'
    else:
        section_label = 'Returns'
    if markdown != '':
        markdown += '\n\n'
    markdown += f'**[{section_label}]**\n'
    for parameter in params_or_rtns:
        markdown += (
            f'\n- `{parameter.name}`: {parameter.type_str}'
            f'\n  - {parameter.description}'
        )
    markdown += '\n\n<hr>'
    return markdown


def extract_reference_values_from_docstring(
        *, docstring: str) -> List[Reference]:
    """
    Extract reference values from a docstring.

    Parameters
    ----------
    docstring : str
        Target docstring.

    Returns
    -------
    reference_values : list of Reference
        Extracted reference values.
    """
    lines: List[str] = docstring.splitlines()
    lines = _remove_blank_lines_from_list(lines=lines)
    is_references_section_range: bool = False
    page_label: str = ''
    url: str = ''
    base_indent_num: int = 0
    reference_values: List[Reference] = []
    for line in lines:
        current_indent_num: int = _get_indent_num_from_line(line=line)
        base_indent_num = _get_base_indent_num_if_not_set(
            line=line, base_indent_num=base_indent_num)
        if _is_target_section_pattern_line(
                line=line,
                section_pattern=_SectionPattern.REFERENCES):
            is_references_section_range = True
            continue
        if _is_skip_target_line(
                is_target_section_range=is_references_section_range,
                line=line):
            continue
        if _is_section_line(line=line):
            break
        if current_indent_num == base_indent_num:
            page_label = _remove_unnecessary_markdown_list_from_line(
                line=line)
            continue
        url = _remove_unnecessary_markdown_list_from_line(line=line)
        url = _remove_noqa(string=url)
        _make_reference_and_append_to_list(
            reference_values=reference_values,
            page_label=page_label, url=url)
        url = ''
    _make_reference_and_append_to_list(
        reference_values=reference_values,
        page_label=page_label, url=url)
    return reference_values


def _remove_noqa(string: str) -> str:
    """
    Remove a noqa comment from a specified string.

    Parameters
    ----------
    string : str
        Target string.

    Returns
    -------
    string : str
        Result string.
    """
    string = string.replace('# noqa', '', 1).strip()
    return string


def _make_reference_and_append_to_list(
        *,
        reference_values: List[Reference],
        page_label: str,
        url: str) -> None:
    """
    Make a reference value and append it to a specified list.

    Parameters
    ----------
    reference_values : list of Reference
        A list to append a reference value.
    page_label : str
        Target reference page label.
    url : str
        Target reference page URL.
    """
    if url == '':
        return
    reference: Reference = Reference(
        page_label=page_label, url=url)
    reference_values.append(reference)


def _remove_unnecessary_markdown_list_from_line(
        *, line: str) -> str:
    """
    Remove unnecessary markdown list string from a line.

    Parameters
    ----------
    line : str
        Target docstring line.

    Returns
    -------
    line : str
        Result docstring line.
    """
    line = line.replace('- ', '', 1)
    line = line.strip()
    return line


def extract_raise_values_from_docstring(*, docstring: str) -> List[Raise]:
    """
    Extract raise values from a docstring.

    Parameters
    ----------
    docstring : str
        Target docstring.

    Returns
    -------
    raise_values : list of Raise
        Extracted raise values.
    """
    lines: List[str] = docstring.splitlines()
    lines = _remove_blank_lines_from_list(lines=lines)
    is_raises_section_range: bool = False
    err_class_name: str = ''
    base_indent_num: int = 0
    description_lines: List[str] = []
    raise_values: List[Raise] = []
    for line in lines:
        current_indent_num: int = _get_indent_num_from_line(line=line)
        base_indent_num = _get_base_indent_num_if_not_set(
            line=line, base_indent_num=base_indent_num)
        if _is_target_section_pattern_line(
                line=line,
                section_pattern=_SectionPattern.RAISES):
            is_raises_section_range = True
            continue
        if _is_skip_target_line(
                is_target_section_range=is_raises_section_range,
                line=line):
            continue
        if _is_section_line(line=line):
            break
        if current_indent_num == base_indent_num:
            _make_raise_description_and_append_to_list(
                raise_values=raise_values,
                err_class_name=err_class_name,
                description_lines=description_lines,
            )
            err_class_name = line.strip()
            continue

        line = _append_br_tag_and_replace_symbol_if_first_char_is_hyphen(
            line=line)
        description_lines.append(line)
    _make_raise_description_and_append_to_list(
        raise_values=raise_values,
        err_class_name=err_class_name,
        description_lines=description_lines,
    )
    return raise_values


def _append_br_tag_and_replace_symbol_if_first_char_is_hyphen(
        line: str) -> str:
    """
    Append a break tag and replace the hyphen symbol
    if the first character is the hyphen symbol.

    Parameters
    ----------
    line : str
        Target docstring line.

    Returns
    -------
    line : str
        Replaced docstring line.
    """
    first_char: str = ''
    for char in line:
        if char == ' ':
            continue
        first_char = char
        break
    if first_char == '-' and line.strip().startswith('- '):
        line = line.replace('- ', '・', 1)
        line = f'<br>{line}'
    return line


def _remove_blank_lines_from_list(*, lines: List[str]) -> List[str]:
    """
    Remove blank lines from a list of lines.

    Parameters
    ----------
    lines : list of str
        Target list of lines.

    Returns
    -------
    result_lines : list of str
        A blank lines removed line list.
    """
    result_lines: List[str] = []
    for line in lines:
        if line.strip() == '':
            continue
        result_lines.append(line)
    return result_lines


def _get_base_indent_num_if_not_set(
        *, line: str, base_indent_num: int) -> int:
    """
    Get a base indent number from a line if it is an
    initial value.

    Parameters
    ----------
    line : str
        Target docstring line.
    base_indent_num : int
        Current base indent number.

    Returns
    -------
    base_indent_num : int
        If the base_indent_num argument is zero, this function
        returns the current line indent number. Otherwise, it
        returns the same value of the base_indent_num argument.
    """
    if base_indent_num != 0:
        return base_indent_num
    current_indent_num: int = _get_indent_num_from_line(line=line)
    return current_indent_num


def _make_raise_description_and_append_to_list(
        *,
        raise_values: List[Raise],
        err_class_name: str,
        description_lines: List[str]) -> None:
    """
    Make a raise value description from a list of lines and
    append raise value to a specified list.

    Notes
    -----
    This function clears a list of description lines.

    Parameters
    ----------
    raise_values : list of Raise
        A list to append a raise value.
    err_class_name : str
        Target error class name.
    description_lines : list of str
        A list of description lines.
    """
    if not description_lines:
        return
    description: str = '\n'.join(description_lines)
    description = _remove_line_breaks_and_unnecessary_spaces(
        text=description)
    raise_: Raise = Raise(
        err_class_name=err_class_name, description=description)
    raise_values.append(raise_)
    description_lines.clear()


def extract_param_or_rtn_values_from_docstring(
        *, target_type: Type[_ParamOrRtn],
        docstring: str) -> List[_ParamOrRtn]:
    """
    Extract parameter or return values from a docstring.

    Parameters
    ----------
    target_type : Type
        Target type of the Parameter or Return.
    docstring : str
        Target docstring.

    Returns
    -------
    param_or_rtn_values : list of Parameter or Return
        Extracted parameter or return values.
    """
    lines: List[str] = docstring.splitlines()
    lines = _remove_blank_lines_from_list(lines=lines)
    is_param_or_rtn_section_range: bool = False
    value_name: str = ''
    value_type_str: str = ''
    base_indent_num: int = 0
    description_lines: List[str] = []
    param_or_rtn_values: List[_ParamOrRtn] = []
    params_or_rtns_section_pattern: _SectionPattern = \
        _get_params_or_rtns_section_pattern_by_type(target_type=target_type)
    for line in lines:
        current_indent_num: int = _get_indent_num_from_line(line=line)
        base_indent_num = _get_base_indent_num_if_not_set(
            line=line, base_indent_num=base_indent_num)
        if _is_target_section_pattern_line(
                line=line,
                section_pattern=params_or_rtns_section_pattern):
            is_param_or_rtn_section_range = True
            continue
        if _is_skip_target_line(
                is_target_section_range=is_param_or_rtn_section_range,
                line=line):
            continue
        if _is_section_line(line=line):
            break
        if current_indent_num == base_indent_num:
            _make_prm_or_rtn_description_and_append_to_list(
                target_type=target_type,
                param_or_rtn_values=param_or_rtn_values,
                value_name=value_name,
                value_type_str=value_type_str,
                description_lines=description_lines,
            )
            value_name, value_type_str = _get_value_name_and_type_from_line(
                line=line)
            continue
        description_lines.append(line)
    _make_prm_or_rtn_description_and_append_to_list(
        target_type=target_type,
        param_or_rtn_values=param_or_rtn_values,
        value_name=value_name,
        value_type_str=value_type_str,
        description_lines=description_lines,
    )
    return param_or_rtn_values


def _is_skip_target_line(
        *, is_target_section_range: bool, line: str) -> bool:
    """
    Get a boolean indicating whether a specified line
    is skipping target or not.

    Parameters
    ----------
    is_target_section_range : bool
        A boolean indicating whether a specified line
        is in a range of target section.
    line : str
        Target docstring line.

    Returns
    -------
    result : bool
        A boolean indicating whether a specified line
        is skipping target or not.
    """
    if not is_target_section_range:
        return True
    if _is_hyphens_line(line=line):
        return True
    return False


def _get_params_or_rtns_section_pattern_by_type(
        *,
        target_type: Type[_ParamOrRtnBase]) -> _SectionPattern:
    """
    Get the parameters or returns section pattern
    of a specified type.

    Parameters
    ----------
    target_type : Parameter or Return
        Target type.

    Returns
    -------
    pattern : _SectionPattern
        Target section pattern.

    Raises
    ------
    ValueError
        If a provided argument is an invalid target's type.
    """
    if target_type == Parameter:
        return _SectionPattern.PARAMETERS
    if target_type == Return:
        return _SectionPattern.RETURNS
    raise ValueError(
        f'Invalid type argument is provided: {target_type}')


def _make_prm_or_rtn_description_and_append_to_list(
        *,
        target_type: Type[_ParamOrRtn],
        param_or_rtn_values: List[_ParamOrRtn],
        value_name: str,
        value_type_str: str,
        description_lines: List[str]) -> None:
    """
    Make a parameter or return value description from a list of
    lines and append parameter or return value to a specified list.

    Notes
    -----
    This function clears a list of description lines.

    Parameters
    ----------
    target_type : Type
        Target type of the Parameter or Return.
    param_or_rtn_values : lisf of _ParamOrRtnBase
        A list to append a parameter or return value.
    value_name : str
        Parameter or return value name.
    value_type_str : str
        Parameter or return type name.
    description_lines : list of str
        A list of description lines.
    """
    if not description_lines:
        return
    description: str = '\n'.join(description_lines)
    description = _remove_line_breaks_and_unnecessary_spaces(
        text=description)
    param_or_rtn: _ParamOrRtn = target_type(
        name=value_name, type_str=value_type_str,
        description=description)
    param_or_rtn_values.append(param_or_rtn)
    description_lines.clear()


def _get_indent_num_from_line(*, line: str) -> int:
    """
    Get an indentation number from a specified docstring line.

    Parameters
    ----------
    line : str
        Target docstring line.

    Returns
    -------
    indent_num : int
        Indent number of a specified docstring line.
    """
    spaces: int = 0
    for char in line:
        if char != ' ':
            break
        spaces += 1
    indent_num: int = spaces // 4
    return indent_num


def _get_value_name_and_type_from_line(*, line: str) -> Tuple[str, str]:
    """
    Get a parameter or return value and type from
    a specified line.

    Parameters
    ----------
    line : str
        Target docstring line.

    Returns
    -------
    value_name : str
        Target parameter or return value name.
    type_name : str
        Target parameter or return value type name.
    """
    if ':' not in line:
        return '', ''
    splitted: List[str] = line.split(':', maxsplit=1)
    value_name: str = splitted[0].strip()
    type_name: str = splitted[1].strip()
    return value_name, type_name


def _is_hyphens_line(*, line: str) -> bool:
    """
    Get a boolean indicating whether a specified line is
    a hyphens line or not.

    Parameters
    ----------
    line : str
        Target docstring line.

    Returns
    -------
    result : bool
        If a specified line is a hyphens line, this function
        returns True.
    """
    match: Optional[Match] = re.search(
        pattern=_HYPHENS_LINE_PATTERN, string=line)
    if match is None:
        return False
    return True


def _is_target_section_pattern_line(
        *,
        line: str,
        section_pattern: _SectionPattern) -> bool:
    """
    Get a boolean indicating whether a specified line
    matches a target section pattern or not.

    Parameters
    ----------
    line : str
        Target docstring line.
    section_pattern : _SectionPattern
        Target section pattern.

    Returns
    -------
    result : bool
        If a specified line is the parameters section,
        this function returns True.
    """
    match: Optional[Match] = re.search(
        pattern=section_pattern.value, string=line)
    if match is None:
        return False
    return True


def extract_summary_from_docstring(*, docstring: str) -> str:
    """
    Extract a summary text from a docstring.

    Parameters
    ----------
    docstring : str
        Target docstring.

    Returns
    -------
    summary : str
        Extracted summary text.

    Notes
    -----
    This function converts line break to a space.
    """
    lines: List[str] = docstring.splitlines()
    result_lines: List[str] = []
    for line in lines:
        if _is_section_line(line=line):
            break
        line = _append_br_tag_and_replace_symbol_if_first_char_is_hyphen(
            line=line)
        result_lines.append(line)
    summary: str = '\n'.join(result_lines)
    summary = _remove_line_breaks_and_unnecessary_spaces(text=summary)
    return summary


def _remove_line_breaks_and_unnecessary_spaces(*, text: str) -> str:
    """
    Remove line breaks of single and unnecessary spaces
    (e.g., double spaces and leading and trailing spaces).

    Parameters
    ----------
    text : str
        Target text.

    Returns
    -------
    text : str
        Converted text.
    """
    from apysc._string import string_util
    text = text.strip()
    text = text.replace('\n', ' ')
    text = string_util.replace_double_spaces_to_single_space(
        string=text)
    text = text.strip()
    return text


def _is_section_line(*, line: str) -> bool:
    """
    Get a boolean indicating whether a specified docstring line
    is a section line or not.

    Parameters
    ----------
    line : str
        Target docstring line text.

    Returns
    -------
    result : bool
        If a specified docstring line is a section line,
        this function returns True.
    """
    for pattern in _SectionPattern:
        match: Optional[Match] = re.search(
            pattern=pattern.value, string=line)
        if match is None:
            continue
        return True
    return False


def _extract_package_path_and_callable_name_from_path(
        *, docstring_path_comment: str) -> Tuple[str, str]:
    """
    Extract a module or class package path and callable
    name from a specified path comment.

    Parameters
    ----------
    docstring_path_comment : str
        Target docstring path comment.

    Returns
    -------
    module_or_class_package_path : str
        Extracted module or class package path.
        e.g., 'apy.path' or 'any.path.AnyClass'.
    callable_name : str
        Extracted callable name.
    """
    path: str = _extract_path_from_docstring_comment(
        docstring_path_comment=docstring_path_comment)
    if '.' not in path:
        return '', ''
    splitted: List[str] = path.rsplit('.', maxsplit=1)
    module_or_class_package_path: str = splitted[0]
    callable_name: str = splitted[1]
    return module_or_class_package_path, callable_name


def _extract_path_from_docstring_comment(
        *, docstring_path_comment: str) -> str:
    """
    Extract a path string from a specified docstring path comment.

    Parameters
    ----------
    docstring_path_comment : str
        Target docstring path comment.

    Returns
    -------
    path : str
        Extracted path string.
    """
    match: Optional[Match] = re.search(
        pattern=DOCSTRING_PATH_COMMENT_PATTERN,
        string=docstring_path_comment)
    if match is None:
        return ''
    path: str = match.group(1)
    path = path.strip()
    return path



"""Implementations of the Line class.
"""


from apysc._display import graphics
from apysc._display.line_base import LineBase
from apysc._geom import point2d
from apysc._html.debug_mode import add_debug_info_setting


class Line(LineBase):
    """
    The line vector graphics class.

    References
    ----------
    - Graphics draw_line interface document
        - https://simon-ritchie.github.io/apysc/graphics_draw_line.html  # noqa
    - Graphics draw_dotted_line interface document
        - https://simon-ritchie.github.io/apysc/graphics_draw_dotted_line.html  # noqa
    - Graphics draw_dashed_line interface document
        - https://simon-ritchie.github.io/apysc/graphics_draw_dashed_line.html  # noqa
    - Graphics draw_round_dotted_line interface document
        - https://simon-ritchie.github.io/apysc/graphics_draw_round_dotted_line.html  # noqa
    - Graphics draw_dash_dotted_line interface document
        - https://simon-ritchie.github.io/apysc/graphics_draw_dash_dotted_line.html # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.line_style(color='#fff', thickness=5)
    >>> line: ap.Line = sprite.graphics.draw_line(
    ...     x_start=50, y_start=50, x_end=150, y_end=50)
    >>> line.line_color
    String('#ffffff')

    >>> line.line_thickness
    Int(5)
    """

    _start_point: 'point2d.Point2D'
    _end_point: 'point2d.Point2D'

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='Line')
    def __init__(
            self, *, parent: 'graphics.Graphics',
            start_point: 'point2d.Point2D',
            end_point: 'point2d.Point2D') -> None:
        """
        Create a line vector graphic.

        Parameters
        ----------
        parent : Graphics
            Graphics instance to link this graphic.
        start_point : Points2D
            Line start point.
        end_point : Points2D
            Line end point.

        References
        ----------
        - Graphics draw_line interface document
            - https://simon-ritchie.github.io/apysc/graphics_draw_line.html  # noqa
        - Graphics draw_dotted_line interface document
            - https://simon-ritchie.github.io/apysc/graphics_draw_dotted_line.html  # noqa
        - Graphics draw_dashed_line interface document
            - https://simon-ritchie.github.io/apysc/graphics_draw_dashed_line.html  # noqa
        - Graphics draw_round_dotted_line interface document
            - https://simon-ritchie.github.io/apysc/graphics_draw_round_dotted_line.html  # noqa
        - Graphics draw_dash_dotted_line interface document
            - https://simon-ritchie.github.io/apysc/graphics_draw_dash_dotted_line.html # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.line_style(color='#fff', thickness=5)
        >>> line: ap.Line = sprite.graphics.draw_line(
        ...     x_start=50, y_start=50, x_end=150, y_end=50)
        >>> line.line_color
        String('#ffffff')

        >>> line.line_thickness
        Int(5)
        """
        from apysc._display.graphics import Graphics
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._validation import geom_validation
        geom_validation.validate_point_2d_type(point=start_point)
        geom_validation.validate_point_2d_type(point=end_point)
        parent_graphics: Graphics = parent
        variable_name: str = expression_variables_util.\
            get_next_variable_name(type_name=var_names.LINE)
        super(Line, self).__init__(
            parent=parent, x=0, y=0, variable_name=variable_name)
        self._start_point = start_point
        self._end_point = end_point
        self._set_initial_basic_values(parent=parent)
        self._append_constructor_expression()
        self._set_line_setting_if_not_none_value_exists(
            parent_graphics=parent_graphics)
        self._set_overflow_visible_setting()

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='Line')
    def _append_constructor_expression(self) -> None:
        """
        Append a constructor expression.
        """
        import apysc as ap
        from apysc._display.stage import get_stage_variable_name
        stage_variable_name: str = get_stage_variable_name()
        points_str: str = self._make_points_expression()
        expression: str = (
            f'var {self.variable_name} = {stage_variable_name}'
            f'\n  .line({points_str})'
            '\n  .attr({'
        )
        expression = self._append_basic_vals_expression(
            expression=expression,
            indent_num=2)
        expression += '\n  });'
        ap.append_js_expression(expression=expression)

    def _make_points_expression(self) -> str:
        """
        Make line start and end expression str.

        Returns
        -------
        expression : str
            Each point expression.
        """
        import apysc as ap
        start_point: ap.Point2D = self._start_point
        end_point: ap.Point2D = self._end_point
        expression: str = (
            f'{start_point.x.variable_name}, '
            f'{start_point.y.variable_name}, '
            f'{end_point.x.variable_name}, '
            f'{end_point.y.variable_name}'
        )
        return expression

    def __repr__(self) -> str:
        """
        Get a string representation of this instance (for the sake of
        debugging).

        Returns
        -------
        repr_str : str
            Type name and variable name will be set
            (e.g., `Line('<variable_name>')`).
        """
        repr_str: str = f"Line('{self.variable_name}')"
        return repr_str



"""This module is for the class definitions of
each path data class.
"""

from apysc._geom.path_bezier_2d import PathBezier2D
from apysc._geom.path_bezier_2d_continual import PathBezier2DContinual
from apysc._geom.path_bezier_3d import PathBezier3D
from apysc._geom.path_bezier_3d_continual import PathBezier3DContinual
from apysc._geom.path_close import PathClose
from apysc._geom.path_horizontal import PathHorizontal
from apysc._geom.path_line_to import PathLineTo
from apysc._geom.path_move_to import PathMoveTo
from apysc._geom.path_vertical import PathVertical


class PathData:
    """
    This class is for the definitions of each path data class.
    """
    MoveTo = PathMoveTo
    LineTo = PathLineTo
    Horizontal = PathHorizontal
    Vertical = PathVertical
    Close = PathClose
    Bezier2D = PathBezier2D
    Bezier2DContinual = PathBezier2DContinual
    Bezier3D = PathBezier3D
    Bezier3DContinual = PathBezier3DContinual



"""This module is for the translation mapping data of the
following document:

Document file: timer_delay.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Timer class delay setting':
    '',

    'This page explains the `Timer` class `delay` argument setting.':
    '',

    '## What argument is this?':
    '',

    'The `delay` argument setting determines the timer tick interval. This setting is a milliseconds unit, so a value of 1000 ticks every 1 second.\n\nThe `int`\\, `float`\\, `Int`\\, `Number`\\, and `FPS` enum can be acceptable.':  # noqa
    '',

    '## Basic usage':
    '',

    'You can set the `delay` parameter at the `Timer` class constructor. The following example sets each timer (`timer_1`, `timer_2`, `timer_3`) and passes the delay values of `100`, `333.3333333`, `16.6666667`.\n\nThe first-timer (`delay` is 100) is called 10 times in a second, and the second one (`delay` is 33.3333333) is 30 times in a second, and the third one (`delay` is 16.6666667) is 60 times.':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    stage_width=350, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50)\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=250, y=50, width=50, height=50)\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The Handler would be called every timer tick.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options[\'rectangle\']\n    rectangle.rotation_around_center += 1\n\n\noptions: _RectOptions = {\'rectangle\': rectangle_1}\ntimer_1: ap.Timer = ap.Timer(\n    handler=on_timer, delay=100, options=options)\ntimer_1.start()\n\noptions = {\'rectangle\': rectangle_2}\ntimer_2: ap.Timer = ap.Timer(\n    handler=on_timer, delay=33.3333333, options=options)\ntimer_2.start()\n\noptions = {\'rectangle\': rectangle_3}\ntimer_3: ap.Timer = ap.Timer(\n    handler=on_timer, delay=16.6666667, options=options)\ntimer_3.start()\n\nap.save_overall_html(\n    dest_dir_path=\'timer_delay_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/timer_delay_basic_usage/index.html" width="350" height="150"></iframe>':  # noqa
    '',

    '## Set the FPS enum value to the delay argument':
    '',

    'You can also pass the `FPS` (frames per second) enum value to the `delay` argument. For example, if the `FPS.FPS_60` is specified, a timer delay becomes 60 frames per second (16.6666667 milliseconds). Likely, the `FPS.FPS_30` is specified, a timer delay becomes 30 frames per second (33.3333333 milliseconds).':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The Handler would be called every timer tick.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options[\'rectangle\']\n    rectangle.rotation_around_center += 1\n\n\noptions: _RectOptions = {\'rectangle\': rectangle}\ntimer: ap.Timer = ap.Timer(\n    handler=on_timer, delay=ap.FPS.FPS_60,\n    options=options)\ntimer.start()\n\nap.save_overall_html(\n    dest_dir_path=\'timer_delay_fps_enum/\')\n```':  # noqa
    '',

    '<iframe src="static/timer_delay_fps_enum/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## See also':
    '',

    '- [FPS enum](fps.md)':
    '',

    '## Timer constructor API':
    '',

    '<!-- Docstring: apysc._time.timer.Timer.__init__ -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `__init__(self, handler:Callable[[_ForwardRef(\'timer_event.TimerEvent\'), ~_O1], NoneType], delay:Union[int, float, apysc._type.number_value_interface.NumberValueInterface, apysc._time.fps.FPS], *, repeat_count:Union[int, apysc._type.int.Int]=0, options:Union[~_O1, NoneType]=None) -> None`<hr>\n\n**[Interface summary]** Timer class to handle function calling at regular intervals.<hr>\n\n**[Parameters]**\n\n- `handler`: _Handler\n  - A handler would be called at regular intervals.\n- `delay`: Int or int or Number or float or FPS\n  - A delay between each `Handler` calling in a millisecond or FPS value. If an `FPS` value is specified, this value becomes a millisecond calculated with that FPS value (e.g., if the `FPS_60` value is specified, then `delay` becomes 16.6666667).\n- `repeat_count`: Int or int\n  - Max count of a `Handler`\'s calling. A timer stops if the `Handler`\'s calling count has reached this value. If 0 is specified, then a timer loops forever.\n- `options`: dict or None, default None\n  - Optional arguments dictionary to pass a `Handler` callable.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options[\'rectangle\']\n...     rectangle.x += 1\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> options: RectOptions = {\'rectangle\': rectangle}\n>>> _ = ap.Timer(\n...     on_timer, delay=ap.FPS.FPS_60, options=options).start()\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [Timer document](https://simon-ritchie.github.io/apysc/timer.html)\n- [TimerEvent class document](https://simon-ritchie.github.io/apysc/timer_event.html)\n- [FPS enum document](https://simon-ritchie.github.io/apysc/fps.html)\n- [Timer class repeat_count setting](https://simon-ritchie.github.io/apysc/timer_repeat_count.html)\n- [About the handler options\' type document](https://simon-ritchie.github.io/apysc/about_handler_options_type.html)':  # noqa
    '',

    '## delay property API':
    '',

    '<!-- Docstring: apysc._time.timer.Timer.delay -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get a delay value.<hr>\n\n**[Returns]**\n\n- `delay`: Number\n  - A delay value of each `Handler` calling in milliseconds.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_timer(e: ap.TimerEvent, options: dict) -> None:\n...     pass\n>>> timer: ap.Timer = ap.Timer(\n...     on_timer, delay=33.3, repeat_count=50)\n>>> timer.delay\nNumber(33.3)\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: contains.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# contains interface':
    '',

    'This page explains the container class, like the `Graphics`\\, `Sprite`\\, `Stage`) `contains` method interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    'The `contains` interface returns the boolean (`Boolean`) value whether that `Sprite` instance has a given child or not.':  # noqa
    '',

    '## Basic usage':
    '',

    'The following example checks whether the first rectangle is a child of the `Sprite` container. If it is `true`\\, remove the rectangle from the sprite and display a log to the console (please press F12 to display that message).':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_sprite_click(\n        e: ap.MouseEvent[ap.Sprite], options: _RectOptions) -> None:\n    """\n    The handler that the sprite calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    sprite: ap.Sprite = e.this\n    rectangle_1: ap.Rectangle = options[\'rectangle\']\n    condition: ap.Boolean = sprite.graphics.contains(child=rectangle_1)\n    with ap.If(condition):\n        sprite.remove_child(child=rectangle_1)\n        ap.trace(\'Removed the rectangle!\')\n\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=250,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nsprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50)\noptions: _RectOptions = {\'rectangle\': rectangle_1}\nsprite.click(on_sprite_click, options=options)\n\nap.save_overall_html(\n    dest_dir_path=\'sprite_contains_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/sprite_contains_basic_usage/index.html" width="250" height="150"></iframe>':  # noqa
    '',

    '## See also':
    '',

    '- [Add child and remove child interfaces](add_child_and_remove_child.md)':  # noqa
    '',

    '## contains API':
    '',

    '<!-- Docstring: apysc._display.child_interface.ChildInterface.contains -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `contains(self, child:apysc._display.display_object.DisplayObject) -> apysc._type.boolean.Boolean`<hr>\n\n**[Interface summary]** Get a boolean whether this instance contains a specified child.<hr>\n\n**[Parameters]**\n\n- `child`: DisplayObject\n  - Child instance to check.\n\n<hr>\n\n**[Returns]**\n\n- `result`: Boolean\n  - If this instance contains a specified child, this method returns True.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\', alpha=0.5)\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> sprite.graphics.contains(rectangle)\nBoolean(True)\n\n>>> rectangle.remove_from_parent()\n>>> sprite.graphics.contains(rectangle)\nBoolean(False)\n```':  # noqa
    '',

}



"""Implementations to manipulate expression variable name
related interface.

Mainly following interfaces are defined:

- get_next_variable_name: Get next variable name of specified type name.
- append_substitution_expression: Append substitution expression between
    two variables.
"""

from typing import Optional
from typing import Tuple

from apysc._type.variable_name_interface import VariableNameInterface


def get_next_variable_name(*, type_name: str) -> str:
    """
    Get next variable name of specified type name.

    Notes
    -----
    If calling this function multiple times, this interface
    increases a returned number.

    Parameters
    ----------
    type_name : str
        Any type name, e.g., `sp` (`Sprite`).
        If `sp` is specified and there is no `sp` variable
        name in expression data, this interface returns
        the `sp_1`. If the apysc already uses a variable
        name of `sp_1`, this interface returns `sp_2`.

    Returns
    -------
    variable_name : str
        Next variable name.
    """
    next_variable_num: int = _get_next_variable_num(
        type_name=type_name)
    variable_name = _make_variable_name(
        type_name=type_name, variable_num=next_variable_num)
    _save_next_variable_name_count(type_name=type_name)
    return variable_name


def _save_next_variable_name_count(*, type_name: str) -> None:
    """
    Save a next variable name count value.

    Parameters
    ----------
    type_name : str
        Any type name, e.g., `sp`.
    """
    from apysc._expression import expression_data_util
    next_variable_num: int = _get_next_variable_num(
        type_name=type_name)
    table_name: str = expression_data_util.TableName.VARIABLE_NAME_COUNT.value
    query: str = (
        f'DELETE FROM {table_name} '
        f"WHERE type_name = '{type_name}';"
    )
    expression_data_util.exec_query(sql=query, commit=False)
    query = (
        f'INSERT INTO {table_name}'
        '(type_name, count) '
        f"VALUES('{type_name}', {next_variable_num});"
    )
    expression_data_util.exec_query(sql=query)


def _make_variable_name(*, type_name: str, variable_num: int) -> str:
    """
    Make variable name from type name and variable num.

    Parameters
    ----------
    type_name : str
        Any type name, e.g., `sprite`.
    variable_num : int
        Target variable number (start from 1).

    Returns
    -------
    variable_name : str
        Variable name that concatenated type name and variable number.
    """
    variable_name: str = f'{type_name}_{variable_num}'
    return variable_name


def _get_next_variable_num(*, type_name: str) -> int:
    """
    Get a next variable number.

    Parameters
    ----------
    type_name : str
        Any type name, e.g., `sprite`.

    Returns
    -------
    next_variable_num : int
        Next variable number (start from 1).
    """
    from apysc._expression import expression_data_util
    table_name: str = expression_data_util.TableName.VARIABLE_NAME_COUNT.value
    query: str = (
        f'SELECT count FROM {table_name} '
        f"WHERE type_name = '{type_name}' LIMIT 1;"
    )
    expression_data_util.exec_query(sql=query)
    result: Optional[Tuple[int]] = expression_data_util.cursor.fetchone()
    if result is None:
        return 1
    next_variable_num: int = result[0] + 1
    return next_variable_num


def append_substitution_expression(
        *, left_value: VariableNameInterface,
        right_value: VariableNameInterface) -> None:
    """
    Append a substitution expression between two variables.

    Parameters
    ----------
    left_value : VariableNameInterface
        Any left value.
    right_value : VariableNameInterface
        Any right value.
    """
    import apysc as ap
    expression: str = (
        f'{left_value.variable_name} = {right_value.variable_name};'
    )
    ap.append_js_expression(expression=expression)


def append_substitution_expression_with_names(
        *, left_variable_name: str,
        right_variable_name: str) -> None:
    """
    Append a substitution expression between two variable names.

    Notes
    -----
    If the left or right variable names are blank, this
    interface skips appending an expression.

    Parameters
    ----------
    left_variable_name : str
        Left-side variable name.
    right_variable_name : str
        Right-side variable name.
    """
    import apysc as ap
    if left_variable_name == '' or right_variable_name == '':
        return
    expression: str = (
        f'{left_variable_name} = {right_variable_name};'
    )
    ap.append_js_expression(expression=expression)



"""The utilities module for each lint and doc's hash file
(used to check whether the files are updated or not).

Mainly following interfaces are defined:

- get_hash_dir_path
    - Get a specified type's hash directory path.
- get_target_file_hash_file_path
    - Get a specified file's hash file path.
- read_target_file_hash
    - Read a specified file's hashed string.
- read_saved_hash
    - Read an already-saved file's hashed string.
- save_target_file_hash
    - Save a target file's current hash.
- save_target_files_hash
    - Save target files' current hash.
- is_file_updated
    - Get a boolean value whether a specified file has been updated.
- remove_not_updated_file_paths
    - Remove not updated files from specified file paths.
"""

import hashlib
import os
from concurrent import futures
from enum import Enum
from multiprocessing import Pool
from multiprocessing import cpu_count
from typing import List

from typing_extensions import TypedDict


class HashType(Enum):

    AUTOFLAKE = 'autoflake'
    ISORT = 'isort'
    AUTOPEP8 = 'autopep8'
    DOCSTRING_SRC = 'docstring_src'
    DOCSTRING_TO_MARKDOWN = 'docstring_to_markdown'
    TRANSLATION_MAPPING_JP = 'translation_mapping_jp'


_HASH_PACKAGE_ROOT_PATH: str = './.lint_and_doc_hash'


def get_hash_dir_path(*, hash_type: HashType) -> str:
    """
    Get a specified type's hash directory path.

    Parameters
    ----------
    hash_type : HashType
        Target hash type.

    Returns
    -------
    dir_path : str
        Target type's hash directory path.

    Notes
    -----
    This interface creates a returned directory path
    if it does not exist.
    """
    dir_path: str = os.path.join(
        _HASH_PACKAGE_ROOT_PATH,
        f'.{hash_type.value}/'
    )
    os.makedirs(dir_path, exist_ok=True)
    return dir_path


def get_target_file_hash_file_path(
        *, file_path: str, hash_type: HashType) -> str:
    """
    Get a specified file's hash file path.

    Parameters
    ----------
    file_path : str
        Target file path.
    hash_type : HashType
        Target hash type.

    Returns
    -------
    file_path : str
        Target hash file path.

    Notes
    -----
    This interface automatically creates a returned file's
    directory path if it does not exist.
    """
    if file_path.startswith('./'):
        file_path = file_path.replace('./', '', 1)
    dir_path: str = get_hash_dir_path(hash_type=hash_type)
    file_path_: str = os.path.join(dir_path, file_path)
    basename: str = os.path.basename(file_path_)
    if '.' in basename:
        extension: str = basename.split('.')[-1]
        file_path_ = file_path_.replace(f'.{extension}', '', 1)
    os.makedirs(os.path.dirname(file_path_), exist_ok=True)
    return file_path_


def read_target_file_hash(*, file_path: str) -> str:
    """
    Read a specified file's hashed string.

    Parameters
    ----------
    file_path : str
        Target file path.

    Returns
    -------
    hashed_string : str
        Hashed file string. If there is no file at
        the specified path, this interface returns
        a blank string.
    """
    from apysc._file import file_util
    if not os.path.isfile(file_path):
        return ''
    file_txt: str = file_util.read_txt(file_path=file_path)
    hashed_string: str = hashlib.sha1(file_txt.encode()).hexdigest()
    return hashed_string


def read_saved_hash(*, file_path: str, hash_type: HashType) -> str:
    """
    Read an already-saved file's hashed string.

    Parameters
    ----------
    file_path : str
        Target file path.
    hash_type : HashType
        Target hash type.

    Returns
    -------
    saved_hash : str
        An already-saved file's hash string. If there
        is no saved hash file, this interface returns
        a blank string.
    """
    from apysc._file import file_util
    file_path_: str = get_target_file_hash_file_path(
        file_path=file_path, hash_type=hash_type)
    if not os.path.isfile(file_path_):
        return ''
    saved_hash: str = file_util.read_txt(file_path=file_path_)
    saved_hash = saved_hash.strip()
    return saved_hash


def save_target_file_hash(*, file_path: str, hash_type: HashType) -> None:
    """
    Save a target file's current hash.

    Parameters
    ----------
    file_path : str
        Target file path.
    hash_type : HashType
        Target hash type.
    """
    from apysc._file import file_util
    hash: str = read_target_file_hash(file_path=file_path)
    if hash == '':
        return
    file_path_: str = get_target_file_hash_file_path(
        file_path=file_path, hash_type=hash_type)
    file_util.save_plain_txt(txt=hash, file_path=file_path_)


def save_target_files_hash(
        *, file_paths: List[str], hash_type: HashType) -> None:
    """
    Save target files' current hash.

    Parameters
    ----------
    file_paths : list of str
        Target file paths.
    hash_type : HashType
        Target hash type.
    """
    workers: int = max(cpu_count() - 1, 1)
    with futures.ProcessPoolExecutor(max_workers=workers) as executor:
        future_list: List[futures.Future] = []
        for file_path in file_paths:
            future = executor.submit(
                save_target_file_hash,
                file_path=file_path,
                hash_type=hash_type)
            future_list.append(future)
        _ = futures.as_completed(fs=future_list)


def is_file_updated(*, file_path: str, hash_type: HashType) -> bool:
    """
    Get a boolean value whether a specified file is changing or not.

    Parameters
    ----------
    file_path : str
        Target file path.
    hash_type : HashType
        Target hash type.

    Returns
    -------
    result : bool
        If a specified file is changing, this interface
        returns True.
    """
    saved_hash: str = read_saved_hash(
        file_path=file_path, hash_type=hash_type)
    current_hash: str = read_target_file_hash(file_path=file_path)
    if saved_hash == current_hash:
        return False
    return True


class _IsFileUpdatedArgs(TypedDict):
    file_path: str
    hash_type: HashType


def _is_file_updated_func_for_multiprocessing(
        args: _IsFileUpdatedArgs) -> bool:
    """
    Wrapper function of the `is_file_updated` function
    for the multiprocessing.

    Parameters
    ----------
    args : _IsFileUpdatedArgs
        Arguments dictionary to pass to the `is_file_updated`
        function.

    Returns
    -------
    result : bool
        If there is an updated file, this interface
        returns True.
    """
    result: bool = is_file_updated(
        file_path=args['file_path'],
        hash_type=args['hash_type'])
    return result


def _create_args_list_for_multiprocessing(
        *, file_paths: List[str],
        hash_type: HashType) -> List[_IsFileUpdatedArgs]:
    """
    Create an arguments list for the multiprocessing.

    Parameters
    ----------
    file_paths : list of str
        Target file paths.
    hash_type : HashType
        Target hash type.

    Returns
    -------
    args_list : list of _IsFileUpdatedArgs
        Created arguments list for the multiprocessing.
    """
    args_list: List[_IsFileUpdatedArgs] = []
    for file_path in file_paths:
        args_list.append({
            'file_path': file_path,
            'hash_type': hash_type,
        })
    return args_list


def remove_not_updated_file_paths(
        *, file_paths: List[str],
        hash_type: HashType) -> List[str]:
    """
    Remove not updated files from specified file paths.

    Parameters
    ----------
    file_paths : list of str
        Target file paths.
    hash_type : HashType
        Target hash type.

    Returns
    -------
    sliced_file_paths : list of str
        After the slicing file paths.
    """
    workers: int = max(cpu_count() - 1, 1)
    args_list: List[_IsFileUpdatedArgs] = \
        _create_args_list_for_multiprocessing(
            file_paths=file_paths, hash_type=hash_type)
    sliced_file_paths: List[str] = []
    with Pool(processes=workers) as p:
        file_updated_bool_list: List[bool] = p.map(
            func=_is_file_updated_func_for_multiprocessing,
            iterable=args_list,
        )
    file_updated: bool
    for i, file_updated in enumerate(file_updated_bool_list):
        if not file_updated:
            continue
        sliced_file_paths.append(file_paths[i])
    return sliced_file_paths



"""Class implementation for the continue.
"""

from apysc._html.debug_mode import add_debug_info_setting


class Continue:
    """
    The loop continue expression class.

    Notes
    -----
    This class can be instantiated in the with loop statement,
    for example, after the `with ap.For(...):` statement.

    References
    ----------
    - Continue document
        - https://simon-ritchie.github.io/apysc/continue.html

    Examples
    --------
    >>> import apysc as ap
    >>> arr: ap.Array = ap.Array(range(3))
    >>> with ap.For(arr) as i:
    ...     with ap.If(i == 1):
    ...         _ = ap.Continue()
    """

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='Continue')
    def __init__(self) -> None:
        """
        The loop continue expression class.

        Notes
        -----
        This class can be instantiated in the with loop statement,
        for example, after the `with ap.For(...):` statement.

        References
        ----------
        - Continue document
            - https://simon-ritchie.github.io/apysc/continue.html

        Examples
        --------
        >>> import apysc as ap
        >>> arr: ap.Array = ap.Array(range(3))
        >>> with ap.For(arr) as i:
        ...     with ap.If(i == 1):
        ...         _ = ap.Continue()
        """
        import apysc as ap
        from apysc._loop import loop_count
        current_loop_count: int = loop_count.get_current_loop_count()
        if current_loop_count == 0:
            err_msg: str = (
                'The `Continue` class can be instantiated in the with '
                'loop statement, for example, after the '
                '`with ap.For(...):` statement.'
            )
            raise Exception(err_msg)
        ap.append_js_expression(expression='continue;')



"""This module is for the translation mapping data of the
following document:

Document file: save_overall_html.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# save_overall_html interface':
    '',

    'This page explains the `save_overall_html` function interface.':
    '',

    '## What interface is this?':
    '',

    'The `save_overall_html` function interface will export the overall HTML and JavaScript files. At the end of the apysc project, this function\'s calling is necessary to export the HTML.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `save_overall_html` function need at least one argument, `dest_dir_path`. This argument determines a directory path to save HTML and JavaScript files.\n\nThe following code example exports the HTML and JavaScript files, and the exported HTML displays the blank stage (150 px width and height).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nap.save_overall_html(\n    dest_dir_path=\'save_overall_html_interface_basic_usage/\')\n```':  # noqa
    '',

    'The preceding code exports the `save_overall_html_interface_basic_usage/index.html` and the other JavaScript library files.\n\n<iframe src="static/save_overall_html_interface_basic_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Minify the HTML':
    '',

    'The `save_overall_html` function has the `minify` optional argument (default is True). This interface minifies an output HTML if this value is `True`\\. The `False` setting is sometimes helpful for debugging.':  # noqa
    '',

    '```py\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nap.save_overall_html(\n    dest_dir_path=\'dest_dir/\',\n    minify=False)\n```':  # noqa
    '',

    '':
    '',

    '## JavaScript libs directory path setting and skip option':
    '',

    'If you want to adjust the JavaScript library paths, set the `js_lib_dir_path` optional argument. This option overrides the JavaScript library paths in an exported HTML (`index.html`).\n\nThis setting is sometimes helpful when you want to export the HTML with specified JavaScript library paths, for instance, the Django library static directory.\n\nAlso, the `skip_js_lib_exporting` option is helpful when you want to skip the already exported js files. This setting skips the JavaScript library exporting.':  # noqa
    '',

    '```py\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nap.save_overall_html(\n    dest_dir_path=\'dest_dir/\',\n    js_lib_dir_path=\'static/js/\',\n    skip_js_lib_exporting=True)\n```':  # noqa
    '',

    'Notes: The `js_lib_dir_path` option does not change the js files exporting destination directory currently.':  # noqa
    '',

    '## Change the HTML file name by the html_file_name option':
    '',

    'If you need to change the output HTML file name, use the `html_file_name` optional argument. This argument changes the HTML file name from `index.html` to any other name.':  # noqa
    '',

    '```py\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nap.save_overall_html(\n    dest_dir_path=\'dest_dir/\',\n    html_file_name=\'chart.html\')\n```':  # noqa
    '',

    '':
    '',

    '## Bundle each JavaScript library to the signle HTML file by the embed_js_libs option':  # noqa
    '',

    'You can bundle each JavaScript library to the single output HTML file by the `embed_js_libs` optional argument. This option is maybe useful when you need to pass the output file to the other members.':  # noqa
    '',

    '```py\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nap.save_overall_html(\n    dest_dir_path=\'dest_dir/\',\n    embed_js_libs=True)\n```':  # noqa
    '',

    '':
    '',

    '## Change the stdout setting by the verbose option':
    '',

    'The `verbose` optional argument changes the exporting stdout behavior. If the specified value is 0, the apysc displays nothing to the stdout will. If 1 or the other values is specified, the apysc displays the stdout.':  # noqa
    '',

    '```py\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nap.save_overall_html(\n    dest_dir_path=\'dest_dir/\',\n    verbose=0)\n```':  # noqa
    '',

    '':
    '',

    '## save_overall_html API':
    '',

    '<!-- Docstring: apysc._html.exporter.save_overall_html -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `save_overall_html(dest_dir_path:str, *, html_file_name:str=\'index.html\', minify:bool=True, js_lib_dir_path:str=\'./\', skip_js_lib_exporting:bool=False, embed_js_libs:bool=False, verbose:int=1) -> None`<hr>\n\n**[Interface summary]** Save the overall HTML and js files under the specified directory path.<hr>\n\n**[Parameters]**\n\n- `dest_dir_path`: str\n  - Destination directory path to save each HTML and js file.\n- `html_file_name`: str, default \'index.html\'\n  - The output HTML file name.\n- `minify`: bool, default True\n  - Boolean value indicates whether minify HTML and js or not. The False setting is helpful when debugging.\n- `js_lib_dir_path`: str, default \'./\'\n  - JavaScript libraries directory path. This setting applies to a JavaScript source path in HTML. If not specified, then set the same directory with HTML. This setting is maybe helpful to set js lib directory, such as Django\'s static (static_collected) directory. This interface recommends setting True value to the `skip_js_lib_exporting` argument if this argument sets.\n- `skip_js_lib_exporting`: bool, default False\n  - If True, this interface does not export JavaScript libraries.\n- `embed_js_libs`: bool, default False\n  - Option to embed the JavaScript libraries script to the output HTML or not. If True, the output HTML becomes enormous, and be only one HTML file. Occasionally, this option is useful when sharing the exported file or using the output file with an iframe tag to avoid the CORS error.\n- `verbose`: int, default 1\n  - The Logging setting. If 0 is specified, this interface does not display a logging message. If 1 or the other value is specified, this interface displays a message usually.\n\n<hr>\n\n**[Notes]**\n\nThis interface empties a specified directory before saving.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> # Do something here...\n>>> ap.save_overall_html(dest_dir_path=\'tmp/output/\')\n```':  # noqa
    '',

}



"""Implementations of Circle class.
"""

from typing import Union

from apysc._display import graphics
from apysc._display.cx_interface import CxInterface
from apysc._display.cy_interface import CyInterface
from apysc._display.line_base import LineBase
from apysc._display.radius_interface import RadiusInterface
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int


class Circle(  # type: ignore
        CxInterface, CyInterface, LineBase, RadiusInterface):
    """
    The circle vector graphics class.

    References
    ----------
    - Graphics draw_circle interface document
        - https://simon-ritchie.github.io/apysc/graphics_draw_circle.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color='#0af')
    >>> circle: ap.Circle = sprite.graphics.draw_circle(
    ...     x=100, y=100, radius=50)
    >>> circle.x
    Int(100)

    >>> circle.y
    Int(100)

    >>> circle.radius
    Int(50)

    >>> circle.fill_color
    String('#00aaff')
    """

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='Circle')
    def __init__(
            self,
            *,
            parent: 'graphics.Graphics',
            x: Union[int, Int],
            y: Union[int, Int],
            radius: Union[int, Int]) -> None:
        """
        Create a circle vector graphic.

        Parameters
        ----------
        parent : Graphics
            Graphics instance to link this graphic.
        x : Int or int
            X-coordinate of the circle center.
        y : Int or int
            Y-coordinate of the circle center.
        radius : Int or int
            Circle radius.

        References
        ----------
        - Graphics draw_circle interface document
            - https://simon-ritchie.github.io/apysc/graphics_draw_circle.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> circle: ap.Circle = sprite.graphics.draw_circle(
        ...     x=100, y=100, radius=50)
        >>> circle.x
        Int(100)

        >>> circle.y
        Int(100)

        >>> circle.radius
        Int(50)

        >>> circle.fill_color
        String('#00aaff')
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._validation import size_validation
        variable_name: str = expression_variables_util.\
            get_next_variable_name(type_name=var_names.CIRCLE)
        super(Circle, self).__init__(
            parent=parent, x=0, y=0, variable_name=variable_name)
        size_validation.validate_size_is_gt_zero(size=radius)
        self._radius = self._get_converted_radius_int(radius=radius)
        self._set_initial_basic_values(parent=parent)
        self._append_constructor_expression()
        self._set_center_coordinates(x=x, y=y)
        self._set_line_setting_if_not_none_value_exists(
            parent_graphics=parent)
        self._set_overflow_visible_setting()

        self._append_applying_new_attr_val_exp(
            new_attr=self._radius, attr_name='radius')
        self._append_attr_to_linking_stack(
            attr=self._radius, attr_name='radius')

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='Circle')
    def _set_center_coordinates(
            self,
            *,
            x: Union[int, Int],
            y: Union[int, Int]) -> None:
        """
        Set a center x-coordinate and a center y-coordinate.

        Parameters
        ----------
        x : Int or int
            X-coordinate of the circle center.
        y : Int or int
            Y-coordinate of the circle center.
        """
        from apysc._converter.to_apysc_val_from_builtin import \
            get_copied_int_from_builtin_val
        self.x = get_copied_int_from_builtin_val(integer=x)
        self.y = get_copied_int_from_builtin_val(integer=y)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='Circle')
    def _append_constructor_expression(self) -> None:
        """
        Append a constructor expression.
        """
        import apysc as ap
        from apysc._display.stage import get_stage_variable_name
        from apysc._type import value_util
        stage_variable_name: str = get_stage_variable_name()
        radius_str: str = value_util.get_value_str_for_expression(
            value=self._radius)
        expression: str = (
            f'var {self.variable_name} = {stage_variable_name}'
            f'\n  .circle({radius_str} * 2)'
            '\n  .attr({'
        )
        expression = self._append_basic_vals_expression(
            expression=expression, indent_num=2)
        expression += '\n  });'
        ap.append_js_expression(expression=expression)

    def __repr__(self) -> str:
        """
        Get a string representation of this instance (for the sake of
        debugging).

        Returns
        -------
        repr_str : str
            Type name and variable name will be set
            (e.g., `Circle('<variable_name>')`).
        """
        repr_str: str = f"Circle('{self.variable_name}')"
        return repr_str



"""This module is for the translation mapping data of the
following document:

Document file: mouseover_and_mouseout.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Mouseover and mouseout interfaces':
    '',

    'This page explains the `mouseover` and `mouseout` interfaces.':
    '',

    '## What interfaces are these?':
    '',

    'The `mouseover` interface binds the event handler. Moreover, this interface calls the handler when a mouse cursor is over on a `DisplayObject` instance. Conversely, the `mouseout` interface also binds and calls the handler when a cursor is out from the `DisplayObject` one.':  # noqa
    '',

    '## See also':
    '',

    'The following page describes the basic mouse event interfaces.\n\n- [Basic mouse event interfaces](mouse_event_basic.md)':  # noqa
    '',

    '## Basic usage of the mouseover and mouseout interfaces':
    '',

    'Each `DisplayObject` instance has the `mouseover` and `mouseout` interfaces, and you can bind handlers by these.\n\nThe following example binds the mouse over and handler and mouse out one to the rectangle. The rectangle color changes when your cursor is over the rectangle. Also, it reverts to the original one when your cursor is outed from the rectangle.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_mouseover(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mouseover.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n\n    # Change the rectangle fill color to magenta.\n    rectangle.fill_color = ap.String(\'#f0a\')\n\n\ndef on_mouseout(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mouseout.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n\n    # Revert the rectangle fill color.\n    rectangle.fill_color = ap.String(\'#0af\')\n\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\n# Bind the mouse over and mouse out event handlers to the rectangle.\nrectangle.mouseover(on_mouseover)\nrectangle.mouseout(on_mouseout)\n\nap.save_overall_html(\n    dest_dir_path=\'mouseover_and_mouseout_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/mouseover_and_mouseout_basic_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Unbind Interfaces':
    '',

    'The `unbind_mouseover` and `unbind_mouseout` interfaces unbind each registered handler from the `DisplayObject`\\.\n\nThe following example unbind handlers in the `on_mouseover` and `on_mouseout` functions so that the interface calls these handlers only once.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_mouseover(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mouseover.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.String(\'#f0a\')\n\n    # Unbind the mouseover handler.\n    rectangle.unbind_mouseover(handler=on_mouseover)\n\n\ndef on_mouseout(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mouseout.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.String(\'#0af\')\n\n    rectangle.unbind_mouseout(handler=on_mouseout)\n\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\nrectangle.mouseover(on_mouseover)\nrectangle.mouseout(on_mouseout)\n\nap.save_overall_html(\n    dest_dir_path=\'mouseover_and_mouseout_unbind_interfaces/\')\n```':  # noqa
    '',

    '<iframe src="static/mouseover_and_mouseout_unbind_interfaces/index.html" width="150" height="150"></iframe>\n\nThere are also existing the `unbind_mouseover_all` and `unbind_mouseover_all` interfaces. These interfaces unbind all the handlers from the target `DisplayObject` instance.':  # noqa
    '',

    '## mouseover API':
    '',

    '<!-- Docstring: apysc._event.mouse_over_interface.MouseOverInterface.mouseover -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `mouseover(self, handler:Callable[[apysc._event.mouse_event.MouseEvent, ~_O], NoneType], *, options:Union[~_O, NoneType]=None) -> str`<hr>\n\n**[Interface summary]** Add mouse over event listener setting.<hr>\n\n**[Parameters]**\n\n- `handler`: _Handler\n  - Callable that would be called when mouse over on this instance.\n- `options`: dict or None, default None\n  - Optional arguments dictionary to be passed to a handler.\n\n<hr>\n\n**[Returns]**\n\n- `name`: str\n  - Handler\'s name.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mouseover(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mouseout(on_mouseover)\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [About the handler options\' type document](https://simon-ritchie.github.io/apysc/about_handler_options_type.html)':  # noqa
    '',

    '## unbind_mouseover API':
    '',

    '<!-- Docstring: apysc._event.mouse_over_interface.MouseOverInterface.unbind_mouseover -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `unbind_mouseover(self, handler:Callable[[apysc._event.mouse_event.MouseEvent, ~_O], NoneType]) -> None`<hr>\n\n**[Interface summary]** Unbind a specified handler\'s mouseover event.<hr>\n\n**[Parameters]**\n\n- `handler`: _Handler\n  - Unbinding target Callable.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mouseover(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n...     rectangle.unbind_mouseover(on_mouseover)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mouseout(on_mouseover)\n```':  # noqa
    '',

    '':
    '',

    '## unbind_mouseover_all API':
    '',

    '<!-- Docstring: apysc._event.mouse_over_interface.MouseOverInterface.unbind_mouseover_all -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `unbind_mouseover_all(self) -> None`<hr>\n\n**[Interface summary]** Unbind all mouseover events.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mouseover(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n...     rectangle.unbind_mouseover_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mouseout(on_mouseover)\n```':  # noqa
    '',

    '':
    '',

    '## mouseout API':
    '',

    '<!-- Docstring: apysc._event.mouse_out_interface.MouseOutInterface.mouseout -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `mouseout(self, handler:Callable[[apysc._event.mouse_event.MouseEvent, ~_O], NoneType], *, options:Union[~_O, NoneType]=None) -> str`<hr>\n\n**[Interface summary]** Add mouse out event listener setting.<hr>\n\n**[Parameters]**\n\n- `handler`: _Handler\n  - Callable that would be called when mouse out on this instance.\n- `options`: dict or None, default None\n  - Optional arguments dictionary to be passed to a handler.\n\n<hr>\n\n**[Returns]**\n\n- `name`: str\n  - Handler\'s name.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mouseout(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mouseout(on_mouseout)\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [About the handler options’ type document](https://simon-ritchie.github.io/apysc/about_handler_options_type.html)':  # noqa
    '',

    '## unbind_mouseout API':
    '',

    '<!-- Docstring: apysc._event.mouse_out_interface.MouseOutInterface.unbind_mouseout -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `unbind_mouseout(self, handler:Callable[[apysc._event.mouse_event.MouseEvent, ~_O], NoneType]) -> None`<hr>\n\n**[Interface summary]** Unbind a specified handler\'s mouse-out event.<hr>\n\n**[Parameters]**\n\n- `handler`: _Handler\n  - Unbinding target Callable.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mouseout(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n...     rectangle.unbind_mouseout(on_mouseout)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mouseout(on_mouseout)\n```':  # noqa
    '',

    '':
    '',

    '## unbind_mouseout_all API':
    '',

    '<!-- Docstring: apysc._event.mouse_out_interface.MouseOutInterface.unbind_mouseout_all -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `unbind_mouseout_all(self) -> None`<hr>\n\n**[Interface summary]** Unbind all mouse out events.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mouseout(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n...     rectangle.unbind_mouseout_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mouseout(on_mouseout)\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: mouse_event_basic.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Basic mouse event interfaces':
    '',

    'This page explains the basic mouse event interfaces, like the `this` attribute.':  # noqa
    '',

    '## Basic binding usage':
    '',

    'Each mouse event binding interface accepts `handler` and `options` arguments. The `handler` argument is each interface\'s callable object when event dispatching.\n\nThe `options` argument is an optional parameter dictionary to be passed to the handler. You can skip this argument.\n\nFor instance, you can set the `click` event as follows:':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\n\nclass _ColorOptions(TypedDict):\n    color: str\n\n\ndef on_rectangle_click(e: ap.MouseEvent, options: _ColorOptions) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    # Change the clicked rectangle color to the passed color.\n    rectangle: ap.Rectangle = e.this\n    color: ap.String = ap.String(options[\'color\'])\n    rectangle.fill_color = color\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\noptions: _ColorOptions = {\'color\': \'#f0a\'}\nrectangle.click(\n    handler=on_rectangle_click, options=options)\n\nap.save_overall_html(\n    dest_dir_path=\'mouse_event_basic_basic_binding_usage/\')\n```':  # noqa
    '',

    'If you click the rectangle, the handler changes the rectangle color to the specified options color.\n\n<iframe src="static/mouse_event_basic_basic_binding_usage/index.html" width="150" height="150"></iframe>\n\nThere are many mouse events binding interfaces, such as the `click`\\, `mousedown`\\, `mouseup`\\, `mouseover`\\, `mouseout`\\, and `mousemove` that the `DisplayObject` instance has.':  # noqa
    '',

    '## Basic unbinding usage':
    '',

    'Each `DisplayObject` instance has the `unbind_<event_name>` interfaces, for example, `unbind_click` or `unbind_mousedown` or something else.\n\nThese interfaces can unbind the single handler setting (remove binding setting).\n\nFor example, the following code unbinds the click event, so the interface doesn\'t call the handler function.':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\n\nclass _ColorOptions(TypedDict):\n    color: str\n\n\ndef on_rectangle_click(e: ap.MouseEvent, options: _ColorOptions) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    # Change the clicked rectangle color to the passed color.\n    rectangle: ap.Rectangle = e.this\n    color: ap.String = ap.String(options[\'color\'])\n    rectangle.fill_color = color\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\noptions: _ColorOptions = {\'color\': \'#f0a\'}\nrectangle.click(\n    handler=on_rectangle_click, options=options)\n\nrectangle.unbind_click(handler=on_rectangle_click)\n\nap.save_overall_html(\n    dest_dir_path=\'mouse_event_basic_basic_unbinding_usage/\')\n```':  # noqa
    '',

    'When you click the following rectangle, nothing happens.\n\n<iframe src="static/mouse_event_basic_basic_unbinding_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Unbind all event handlers':
    '',

    'Sometimes, it is helpful to unbind specific all the events at once. For example, each event interface has the `unbind_<event_name>_all` method (e.g., `unbind_click_all`). It can unbind all event handlers from that instance.\n\nThe following code calls the `unbind_click_all` method and removes all handler settings.':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\n\nclass _ColorOptions(TypedDict):\n    color: str\n\n\ndef change_color_on_rectangle_click(\n        e: ap.MouseEvent, options: _ColorOptions) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle: ap.Rectangle = e.this\n    color: ap.String = ap.String(options[\'color\'])\n    rectangle.fill_color = color\n\n\ndef change_x_on_rectangle_click(\n        e: ap.MouseEvent, options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.x += 50\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\noptions: _ColorOptions = {\'color\': \'#f0a\'}\nrectangle.click(\n    handler=change_color_on_rectangle_click, options=options)\nrectangle.click(handler=change_x_on_rectangle_click)\n\nrectangle.unbind_click_all()\n\nap.save_overall_html(\n    dest_dir_path=\'mouse_event_basic_unbind_all_event_handlers/\')\n```':  # noqa
    '',

    'Nothing happens when clicking the rectangle (no color change and no x-coordinate change).\n\n<iframe src="static/mouse_event_basic_unbind_all_event_handlers/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Handler argument names and types':
    '',

    'Handler function (or method) first argument requires the type of the `MouseEvent`\\.\n\nAlso, a second argument name is required to be `options`\\. This argument type becomes `dict`\\. If you skip options argument specification at binding the event, then this argument becomes a blank dictionary (`{}`).':  # noqa
    '',

    '## MouseEvent this attribute':
    '',

    'The `MouseEvent` instance has the `this` attribute, which becomes an event target instance. So, if you bind the click event to the rectangle instance, the `this` attribute becomes that rectangle instance.':  # noqa
    '',

    '## MouseEvent generic type settings':
    '',

    'Suppose you know that you only use one of the handlers by an instance of a particular type. In that case, you can set generic type settings to the `MouseEvent` type annotation (e.g., `MouseEvent[Rectangle]`).\n\nThis setting is helpful to determine the `this` attribute type, and the type-checking library, such as the `mypy` or `Pylance`\\, checks the instance type.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\n\ndef on_rectangle_mousedown(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mousedown.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    rectangle = e.this\n    rectangle.x += 50\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.mousedown(handler=on_rectangle_mousedown)\n```':  # noqa
    '',

    '':
    '',

    '## MouseEvent stage_x and stage_y attributes':
    '',

    'MouseEvent instance has the `stage_x` and `stage_y` attributes. These attributes are absolute coordinates from the upper-left position of the stage.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=200,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\n\ndef on_mousemove(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mousemove.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    ap.trace(\'stage_x:\', e.stage_x, \'stage_y:\', e.stage_y)\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=100, width=50, height=50)\nrectangle.mousemove(handler=on_mousemove)\n\nap.save_overall_html(\n    dest_dir_path=\'mouse_event_basic_stage_x_and_stage_y\')\n```':  # noqa
    '',

    'If you open the DevTools console on Chrome (press F12) and move the mouse cursor on the following rectangle, you can check the `stage_x` and `stage_y` coordinates. The previous code positions the rectangle at `(50, 100)`, so the `stage_x` becomes the range of 50 to 100, and `stage_y` becomes 100 to 150.\n\n<iframe src="static/mouse_event_basic_stage_x_and_stage_y/index.html" width="150" height="200"></iframe>':  # noqa
    '',

    '## MouseEvent local_x and local_y attributes':
    '',

    'MouseEvent instance also has `local_x` and `local_y` attributes. These attributes are the local coordinates from the event registered instance position.\n\nThe following example shows that local_x and local_y become the coordinates in the rectangle area. Both of the `local_x` and `local_y` become a range of 0 to 50 because the rectangle size is 50-pixel.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\n\ndef on_mousemove(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mousemove.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments.\n    """\n    ap.trace(\'local_x:\', e.local_x, \'local_y:\', e.local_y)\n\n\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.mousemove(handler=on_mousemove)\n\nap.save_overall_html(\n    dest_dir_path=\'mouse_event_basic_local_x_and_local_y\')\n```':  # noqa
    '',

    'Please check on Chrome DevTools (press F12) and move the mouse cursor on the following rectangle.\n\n<iframe src="static/mouse_event_basic_local_x_and_local_y/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## See also':
    '',

    '- [Click interface](click.md)\n- [Mousedown and mouseup interfaces](mousedown_and_mouseup.md)\n- [Mouseover and mouseout interfaces](mouseover_and_mouseout.md)\n- [Mousemove interface](mousemove.md)':  # noqa
    '',

    '## stage_x property API':
    '',

    '<!-- Docstring: apysc._event.mouse_event.MouseEvent.stage_x -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get the x-coordinate of the stage reference.<hr>\n\n**[Returns]**\n\n- `x`: Int\n  - x-coordinate.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mousedown(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     stage_x: ap.Int = e.stage_x\n...     # Do something here with the coordinate.\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mousedown(on_mousedown)\n```':  # noqa
    '',

    '':
    '',

    '## stage_y property API':
    '',

    '<!-- Docstring: apysc._event.mouse_event.MouseEvent.stage_y -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get the y-coordinate of the stage reference.<hr>\n\n**[Returns]**\n\n- `y`: Int\n  - y-coordinate.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mousedown(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     stage_y: ap.Int = e.stage_y\n...     # Do something here with the coordinate.\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mousedown(on_mousedown)\n```':  # noqa
    '',

    '':
    '',

    '## local_x property API':
    '',

    '<!-- Docstring: apysc._event.mouse_event.MouseEvent.local_x -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get a local x-coordinate event listening instance. For example, this value becomes x-coordinate from Sprite\'s left-end position by clicking a Sprite instance.<hr>\n\n**[Returns]**\n\n- `x`: Int\n  - x-coordinate.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mousedown(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     local_x: ap.Int = e.local_x\n...     # Do something here with the coordinate.\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mousedown(on_mousedown)\n```':  # noqa
    '',

    '':
    '',

    '## local_y property API':
    '',

    '<!-- Docstring: apysc._event.mouse_event.MouseEvent.local_y -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get the local y-coordinate of the event listening instance. For example, this value becomes y-coordinate from Sprite\'s top-end position by clicking a Sprite instance.<hr>\n\n**[Returns]**\n\n- `y`: Int\n  - y-coordinate.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mousedown(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     local_y: ap.Int = e.local_y\n...     # Do something here with the coordinate.\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mousedown(on_mousedown)\n```':  # noqa
    '',

    '':
    '',

    '# this property API':
    '',

    '<!-- Docstring: apysc._event.event.Event.this -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get an instance of listening to this event.<hr>\n\n**[Returns]**\n\n- `this`: VariableNameInterface\n  - Instance that listening this event.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_custom_event(\n...         e: ap.Event[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> e: ap.Event = ap.Event(this=rectangle)\n>>> _ = rectangle.bind_custom_event(\n...     custom_event_type=\'my_custom_event\',\n...     handler=on_custom_event, e=e)\n>>> # Do something here and then trigger the custom event\n>>> rectangle.trigger_custom_event(\n...     custom_event_type=\'my_custom_event\')\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: animation_width_and_height.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# animation_width and animation_height interfaces':
    '',

    'This page explains the `animation_width` and `animation_height` interfaces.':  # noqa
    '',

    '## What interfaces are these?':
    '',

    'The `animation_width` method interface creates an `AnimationWidth` instance. You can animate object width with it.\n\nSimilarly, the `animation_height` method interface creates an `AnimationHeight` instance. You can animate object height with it.\n\nThese interfaces exist on some `DisplayObject` instances, such as the `Rectangle` class.':  # noqa
    '',

    '## Basic usage':
    '',

    'The following example sets the width animation (from 50 to 100) with the `animation_width` method:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nDURATION: int = 1000\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_width: ap.AnimationWidth = rectangle.animation_width(\n        width=50, duration=DURATION, easing=EASING)\n    animation_width.animation_complete(on_animation_complete_2)\n    animation_width.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_width: ap.AnimationWidth = rectangle.animation_width(\n        width=100, duration=DURATION, easing=EASING)\n    animation_width.animation_complete(on_animation_complete_1)\n    animation_width.start()\n\n\nap.Stage(\n    stage_width=200, stage_height=150,\n    background_color=\'#333\', stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#00aaff\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nanimation_width: ap.AnimationWidth = rectangle.animation_width(\n    width=100, duration=DURATION, easing=EASING)\nanimation_width.animation_complete(on_animation_complete_1)\nanimation_width.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./animation_width_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/animation_width_basic_usage/index.html" width="200" height="150"></iframe>\n\nSimilarly, the following example animates the height with the `animation_height` method:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nDURATION: int = 1000\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_height: ap.AnimationHeight = rectangle.animation_height(\n        height=50, duration=DURATION, easing=EASING)\n    animation_height.animation_complete(on_animation_complete_2)\n    animation_height.start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_height: ap.AnimationHeight = rectangle.animation_height(\n        height=100, duration=DURATION, easing=EASING)\n    animation_height.animation_complete(on_animation_complete_1)\n    animation_height.start()\n\n\nap.Stage(\n    stage_width=150, stage_height=200,\n    background_color=\'#333\', stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#00aaff\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nanimation_height: ap.AnimationHeight = rectangle.animation_height(\n    height=100, duration=DURATION, easing=EASING)\nanimation_height.animation_complete(on_animation_complete_1)\nanimation_height.start()\n\nap.save_overall_html(\n    dest_dir_path=\'./animation_height_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/animation_height_basic_usage/index.html" width="150" height="200"></iframe>':  # noqa
    '',

    '## Notes for the Ellipse instance':
    '',

    'The ellipse instance\'s `animation_width` and `animation_height` interfaces will return an `AnimationWidthForEllipse` and `AnimationHeightForEllipse` instance instead of an `AnimationWidth` instance, for the reason of the internal implementation, as follows:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=200,\n    background_color=\'#333\', stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#00aaff\')\nellipse: ap.Ellipse = sprite.graphics.draw_ellipse(\n    x=100, y=100, width=100, height=100)\nanimation_width: ap.AnimationWidthForEllipse = ellipse.animation_width(\n    width=200, duration=1000)\nanimation_width.start()\n```':  # noqa
    '',

    '':
    '',

    '## animation_width API':
    '',

    '<!-- Docstring: apysc._animation.animation_width_interface.AnimationWidthInterface.animation_width -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `animation_width(self, width:Union[int, apysc._type.int.Int], *, duration:Union[int, apysc._type.int.Int]=3000, delay:Union[int, apysc._type.int.Int]=0, easing:apysc._animation.easing.Easing=<Easing.LINEAR: \'function(x) {return x;}\'>) -> apysc._animation.animation_width.AnimationWidth`<hr>\n\n**[Interface summary]** Set the width animation setting.<hr>\n\n**[Parameters]**\n\n- `width`: Int or int\n  - The final width of the animation.\n- `duration`: Int or int, default 3000\n  - Milliseconds before an animation ends.\n- `delay`: Int or int, default 0\n  - Milliseconds before an animation starts.\n- `easing`: Easing, default Easing.LINEAR\n  - Easing setting.\n\n<hr>\n\n**[Returns]**\n\n- `animation_width`: AnimationWidth\n  - Created animation setting instance.\n\n<hr>\n\n**[Notes]**\n\nTo start this animation, you need to call the `start` method of the returned instance.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.animation_width(\n...     width=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [Animation interfaces duration setting document](https://simon-ritchie.github.io/apysc/animation_duration.html)\n- [Animation interfaces delay setting document](https://simon-ritchie.github.io/apysc/animation_delay.html)\n- [Each animation interface return value document](https://simon-ritchie.github.io/apysc/animation_return_value.html)\n- [Sequential animation setting document](https://simon-ritchie.github.io/apysc/sequential_animation.html)\n- [animation_parallel interface document](https://simon-ritchie.github.io/apysc/animation_parallel.html)\n- [Easing enum document](https://simon-ritchie.github.io/apysc/easing_enum.html)':  # noqa
    '',

    '## animation_height API':
    '',

    '<!-- Docstring: apysc._animation.animation_height_interface.AnimationHeightInterface.animation_height -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `animation_height(self, height:Union[int, apysc._type.int.Int], *, duration:Union[int, apysc._type.int.Int]=3000, delay:Union[int, apysc._type.int.Int]=0, easing:apysc._animation.easing.Easing=<Easing.LINEAR: \'function(x) {return x;}\'>) -> apysc._animation.animation_height.AnimationHeight`<hr>\n\n**[Interface summary]** Set the height animation setting.<hr>\n\n**[Parameters]**\n\n- `height`: Int or int\n  - The final height of the animation.\n- `duration`: Int or int, default 3000\n  - Milliseconds before an animation ends.\n- `delay`: Int or int, default 0\n  - Milliseconds before an animation starts.\n- `easing`: Easing, default Easing.LINEAR\n  - Easing setting.\n\n<hr>\n\n**[Returns]**\n\n- `animation_height`: AnimationHeight\n  - Created animation setting instance.\n\n<hr>\n\n**[Notes]**\n\nTo start this animation, you need to call the `start` method of the returned instance.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.animation_height(\n...     height=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [Animation interfaces duration setting document](https://simon-ritchie.github.io/apysc/animation_duration.html)\n- [Animation interfaces delay setting document](https://simon-ritchie.github.io/apysc/animation_delay.html)\n- [Each animation interface return value document](https://simon-ritchie.github.io/apysc/animation_return_value.html)\n- [Sequential animation setting document](https://simon-ritchie.github.io/apysc/sequential_animation.html)\n- [animation_parallel interface document](https://simon-ritchie.github.io/apysc/animation_parallel.html)\n- [Easing enum document](https://simon-ritchie.github.io/apysc/easing_enum.html)':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: display_object.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# DisplayObject':
    '',

    'This page explains the `DisplayObject` class.':
    '',

    '## What is the DisplayObject?':
    '',

    'The `DisplayObject` is the apysc base class for each display class, such as  `Sprite`\\, `Rectangle`\\, `Circle`\\, or something else.\n\nYou can verify the `DisplayObject` inheritance with each instance by the `isinstance` function.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\ncircle: ap.Circle = sprite.graphics.draw_circle(x=100, y=100, radius=100)\n\n# Verify each instance type.\nassert isinstance(sprite, ap.DisplayObject)\nassert isinstance(circle, ap.DisplayObject)\n```':  # noqa
    '',

    'The apysc uses this class for the basic interfaces or the creating the new display object with the `DisplayObject` inheritance.\n\nThe `DisplayObject` class has the basic interfaces, like `x`\\, `y`\\, `visible`\\, each mouse event binding, or others. The following page explains these interfaces one by one.':  # noqa
    '',

    '## See also':
    '',

    '- [DisplayObject class x and y interfaces](display_object_x_and_y.md)\n- [DisplayObject class parent interfaces](display_object_parent.md)\n- [DisplayObject class visible interface](display_object_visible.md)\n- [DisplayObject class mouse event binding interfaces](display_object_mouse_event.md)':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: array_insert_and_insert_at.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Array class insert and insert_at interfaces':
    '',

    'This page explains the `Array` class `insert` and `insert_at` method interfaces.':  # noqa
    '',

    '## What interfaces are these?':
    '',

    'The `insert` and `insert_at` method interfaces append any value at the specified index. Both interfaces behave the same way (the `insert` is the alias of the `insert_at`).':  # noqa
    '',

    '## Basic usage':
    '',

    'The `insert` and `insert_at` have the same argument, the `index` and `value`\\. The `index` argument accepts an `int` and `Int` value.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 4])\narr.insert(index=1, value=2)\nassert arr == [1, 2, 4]\n\nindex: ap.Int = ap.Int(2)\narr.insert_at(index=index, value=3)\nassert arr == [1, 2, 3, 4]\n```':  # noqa
    '',

    '':
    '',

    '## insert API':
    '',

    '<!-- Docstring: apysc._type.array.Array.insert -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `insert(self, index:Union[int, apysc._type.int.Int], value:~T) -> None`<hr>\n\n**[Interface summary]** Insert value to this array at a specified index. This interface behaves the same `insert_at` method.<hr>\n\n**[Parameters]**\n\n- `index`: Int or int\n  - Index to append value.\n- `value`: *\n  - Any value to append.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 3])\n>>> arr.insert(index=1, value=2)\n>>> arr\nArray([1, 2, 3])\n```':  # noqa
    '',

    '':
    '',

    '## insert_at API':
    '',

    '<!-- Docstring: apysc._type.array.Array.insert_at -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `insert_at(self, index:Union[int, apysc._type.int.Int], value:~T) -> None`<hr>\n\n**[Interface summary]** Insert value to this array at a specified index. This interface behaves the same `insert` method.<hr>\n\n**[Parameters]**\n\n- `index`: Int or int\n  - Index to append value.\n- `value`: *\n  - Any value to append.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 3])\n>>> arr.insert_at(index=1, value=2)\n>>> arr\nArray([1, 2, 3])\n```':  # noqa
    '',

}
